@@ -16,7 +16,7 @@
  * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015-2019 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2024 Nanook Consulting  All rights reserved.
  * Copyright (c) 2021      FUJITSU LIMITED.  All rights reserved.
  * $COPYRIGHT$
  *
@@ -52,13 +52,19 @@
 #include "src/util/pmix_printf.h"
 #include "src/util/pmix_show_help.h"
 
+/*
+ * global variables
+ */
+bool pmix_mca_base_var_initialized = false;
+pmix_list_t pmix_mca_base_var_file_values = PMIX_LIST_STATIC_INIT;
+pmix_list_t pmix_mca_base_var_override_values = PMIX_LIST_STATIC_INIT;
+
 /*
  * local variables
  */
 static pmix_pointer_array_t pmix_mca_base_vars;
 static char *home = NULL;
 static char *cwd = NULL;
-bool pmix_mca_base_var_initialized = false;
 static char *force_agg_path = NULL;
 static char *pmix_mca_base_var_files = NULL;
 static char **pmix_mca_base_var_file_list = NULL;
@@ -66,8 +72,6 @@ static char *pmix_mca_base_var_override_file = NULL;
 static char *pmix_mca_base_var_file_prefix = NULL;
 static char *pmix_mca_base_param_file_path = NULL;
 static bool pmix_mca_base_var_suppress_override_warning = false;
-static pmix_list_t pmix_mca_base_var_file_values = PMIX_LIST_STATIC_INIT;
-static pmix_list_t pmix_mca_base_var_override_values = PMIX_LIST_STATIC_INIT;
 static int pmix_mca_base_var_count = 0;
 static pmix_hash_table_t pmix_mca_base_var_index_hash = PMIX_HASH_TABLE_STATIC_INIT;
 
@@ -198,9 +202,9 @@ static char *append_filename_to_list(const char *filename)
 {
     int i, count;
 
-    (void) pmix_argv_append_unique_nosize(&pmix_mca_base_var_file_list, filename);
+    (void) PMIx_Argv_append_unique_nosize(&pmix_mca_base_var_file_list, filename);
 
-    count = pmix_argv_count(pmix_mca_base_var_file_list);
+    count = PMIx_Argv_count(pmix_mca_base_var_file_list);
 
     for (i = count - 1; i >= 0; --i) {
         if (0 == strcmp(pmix_mca_base_var_file_list[i], filename)) {
@@ -289,7 +293,7 @@ static void resolve_relative_paths(char **file_prefix, char *file_path, bool rel
 int pmix_mca_base_var_cache_files(bool rel_path_search)
 {
     char *tmp;
-    int ret;
+    int ret = 0;
 
     /* We may need this later */
     home = (char *) pmix_home_directory(geteuid());
@@ -321,8 +325,7 @@ int pmix_mca_base_var_cache_files(bool rel_path_search)
         return PMIX_ERR_OUT_OF_RESOURCE;
     }
 
-    /* Initialize a parameter that says where MCA param files can be found.
-       We may change this value so set the scope to PMIX_MCA_BASE_VAR_SCOPE_READONLY */
+    /* Initialize a parameter that says where MCA param files can be found */
     tmp = pmix_mca_base_var_files;
     ret = pmix_mca_base_var_register("pmix", "mca", "base", "param_files",
                                      "Path for MCA "
@@ -356,7 +359,8 @@ int pmix_mca_base_var_cache_files(bool rel_path_search)
     }
 
     /* Disable reading MCA parameter files. */
-    if (0 == strcmp(pmix_mca_base_var_files, "none")) {
+    if (NULL == pmix_mca_base_var_files ||
+        0 == strcmp(pmix_mca_base_var_files, "none")) {
         return PMIX_SUCCESS;
     }
 
@@ -591,13 +595,14 @@ static int var_set_from_string(pmix_mca_base_var_t *var, char *src)
     pmix_mca_base_var_storage_t *dst = var->mbv_storage;
     uint64_t int_value = 0;
     int ret;
+    bool *boolcastme;
+    char *tmp;
 
     switch (var->mbv_type) {
     case PMIX_MCA_BASE_VAR_TYPE_INT:
     case PMIX_MCA_BASE_VAR_TYPE_UNSIGNED_INT:
     case PMIX_MCA_BASE_VAR_TYPE_UNSIGNED_LONG:
     case PMIX_MCA_BASE_VAR_TYPE_UNSIGNED_LONG_LONG:
-    case PMIX_MCA_BASE_VAR_TYPE_BOOL:
     case PMIX_MCA_BASE_VAR_TYPE_SIZE_T:
         ret = int_from_string(src, var->mbv_enumerator, &int_value);
         if (PMIX_ERR_VALUE_OUT_OF_BOUNDS == ret
@@ -632,12 +637,26 @@ static int var_set_from_string(pmix_mca_base_var_t *var, char *src)
         } else if (PMIX_MCA_BASE_VAR_TYPE_SIZE_T == var->mbv_type) {
             size_t *castme = (size_t *) var->mbv_storage;
             *castme = (size_t) int_value;
-        } else if (PMIX_MCA_BASE_VAR_TYPE_BOOL == var->mbv_type) {
-            bool *castme = (bool *) var->mbv_storage;
-            *castme = !!int_value;
         }
 
         return ret;
+
+    case PMIX_MCA_BASE_VAR_TYPE_BOOL:
+        boolcastme = (bool *) var->mbv_storage;
+        int_value = strtoull(src, &tmp, 0);
+        if (tmp[0] == '\0') {
+            *boolcastme = !!int_value;
+        } else {
+            if (0 == strncasecmp(src, "true", strlen(src))) {
+                *boolcastme = true;
+            } else if (0 == strncasecmp(src, "false", strlen(src))) {
+                *boolcastme = false;
+            } else {
+                return PMIX_ERR_VALUE_OUT_OF_BOUNDS;
+            }
+        }
+        return PMIX_SUCCESS;
+
     case PMIX_MCA_BASE_VAR_TYPE_DOUBLE:
         dst->lfval = strtod(src, NULL);
         break;
@@ -902,7 +921,7 @@ int pmix_mca_base_var_build_env(char ***env, int *num_env)
 
 cleanup:
     if (*num_env > 0) {
-        pmix_argv_free(*env);
+        PMIx_Argv_free(*env);
         *num_env = 0;
         *env = NULL;
     }
@@ -940,7 +959,7 @@ int pmix_mca_base_var_finalize(void)
         pmix_mca_base_var_count = 0;
 
         if (NULL != pmix_mca_base_var_file_list) {
-            pmix_argv_free(pmix_mca_base_var_file_list);
+            PMIx_Argv_free(pmix_mca_base_var_file_list);
         }
         pmix_mca_base_var_file_list = NULL;
 
@@ -967,9 +986,9 @@ static int fixup_files(char **file_list, char *path, bool rel_path_search, char
     int mode = R_OK; /* The file exists, and we can read it */
     int count, i, argc = 0;
 
-    search_path = pmix_argv_split(path, PMIX_ENV_SEP);
-    files = pmix_argv_split(*file_list, sep);
-    count = pmix_argv_count(files);
+    search_path = PMIx_Argv_split(path, PMIX_ENV_SEP);
+    files = PMIx_Argv_split(*file_list, sep);
+    count = PMIx_Argv_count(files);
 
     rel_path = force_agg_path ? force_agg_path : cwd;
 
@@ -1012,21 +1031,21 @@ static int fixup_files(char **file_list, char *path, bool rel_path_search, char
 
     if (PMIX_SUCCESS == exit_status) {
         free(*file_list);
-        *file_list = pmix_argv_join(argv, sep);
+        *file_list = PMIx_Argv_join(argv, sep);
     }
 
     if (NULL != files) {
-        pmix_argv_free(files);
+        PMIx_Argv_free(files);
         files = NULL;
     }
 
     if (NULL != argv) {
-        pmix_argv_free(argv);
+        PMIx_Argv_free(argv);
         argv = NULL;
     }
 
     if (NULL != search_path) {
-        pmix_argv_free(search_path);
+        PMIx_Argv_free(search_path);
         search_path = NULL;
     }
 
@@ -1035,14 +1054,19 @@ static int fixup_files(char **file_list, char *path, bool rel_path_search, char
 
 static int read_files(char *file_list, pmix_list_t *file_values, char sep)
 {
-    char **tmp = pmix_argv_split(file_list, sep);
-    int i, count;
+    char **tmp;
+    int i, count, rc;
 
+    if (NULL == file_list) {
+        return PMIX_SUCCESS;
+    }
+
+    tmp = PMIx_Argv_split(file_list, sep);
     if (!tmp) {
         return PMIX_ERR_OUT_OF_RESOURCE;
     }
 
-    count = pmix_argv_count(tmp);
+    count = PMIx_Argv_count(tmp);
 
     /* Iterate through all the files passed in -- read them in reverse
        order so that we preserve unix/shell path-like semantics (i.e.,
@@ -1050,10 +1074,15 @@ static int read_files(char *file_list, pmix_list_t *file_values, char sep)
 
     for (i = count - 1; i >= 0; --i) {
         char *file_name = append_filename_to_list(tmp[i]);
-        pmix_mca_base_parse_paramfile(file_name, file_values);
+        rc = pmix_mca_base_parse_paramfile(file_name, file_values);
+        if (PMIX_SUCCESS != rc && PMIX_ERR_NOT_FOUND != rc) {
+            // it is okay if the file isn't found
+            PMIx_Argv_free(tmp);
+            return rc;
+        }
     }
 
-    pmix_argv_free(tmp);
+    PMIx_Argv_free(tmp);
 
     pmix_mca_base_internal_env_store();
 
